use axum::{
    body::Body,
    extract::{FromRequest, FromRequestParts, Query},
};
use http::Request;
use i18n::{message, message_ids::MessageId};
use validator::Validate;

use crate::{context::Context, model::response::ResErr};

pub struct QueryParams<T>(pub T);

impl<S, T> FromRequest<S> for QueryParams<T>
where
    S: Send + Sync,
    T: Validate,
    Query<T>: FromRequestParts<S>,
{
    type Rejection = ResErr;

    async fn from_request(req: http::Request<Body>, state: &S) -> Result<Self, Self::Rejection> {
        let (parts, body) = req.into_parts();
        let query = Query::<T>::from_request(Request::from_parts(parts.clone(), body), state).await;

        let context: &Context = parts.extensions.get().unwrap();

        if let Ok(Query(data)) = query {
            match crate::model::validator::validate_params(&data, context.get_lang_tag()) {
                Ok(_) => Ok(Self(data)),
                Err(err) => Err(err),
            }
        } else {
            let err = Err(ResErr::params(message!(
                context.get_lang_tag(),
                MessageId::InvalidParams
            )));
            err
        }
    }
}
